@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -67,12 +67,12 @@
 
 package RT::Dashboards;
 
-use RT::Dashboard;
-
 use strict;
 use warnings;
 use base 'RT::SharedSettings';
 
+use RT::Dashboard;
+
 sub RecordClass {
     return 'RT::Dashboard';
 }
@@ -104,8 +104,20 @@ sub LimitToPrivacy {
     }
 }
 
-sub ColumnMapClassName {
-    return 'RT__Dashboard';
+=head2 SortDashboards
+
+Sort the list of dashboards. The default is to sort alphabetically.
+
+=cut
+
+sub SortDashboards {
+    my $self = shift;
+
+    # Work directly with the internal data structure since Dashboards
+    # aren't fully backed by a DB table and can't support typical OrderBy, etc.
+    my @sorted = sort { lcfirst($a->Name) cmp lcfirst($b->Name) } @{$self->{'objects'}};
+    @{$self->{'objects'}} = @sorted;
+    return;
 }
 
 RT::Base->_ImportOverlays();
