@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -109,6 +109,7 @@ problem in Perl that hides the top-level optree from L<Devel::Cover>.
 
 our $port;
 our @SERVERS;
+my @ports; # keep track of all the random ports we used
 
 BEGIN {
     delete $ENV{$_} for qw/LANGUAGE LC_ALL LC_MESSAGES LANG/;
@@ -146,7 +147,7 @@ sub import {
 
     $class->bootstrap_tempdir;
 
-    $class->bootstrap_port;
+    $port = $class->find_idle_port;
 
     $class->bootstrap_plugins_paths( %args );
 
@@ -229,7 +230,7 @@ sub db_requires_no_dba {
     return 1 if $db_type eq 'SQLite';
 }
 
-sub bootstrap_port {
+sub find_idle_port {
     my $class = shift;
 
     my %ports;
@@ -245,6 +246,7 @@ sub bootstrap_port {
 
     # Pick a random port, checking that the port isn't in our in-use
     # list, and that something isn't already listening there.
+    my $port;
     {
         $port = 1024 + int rand(10_000) + $$ % 1024;
         redo if $ports{$port};
@@ -271,6 +273,8 @@ sub bootstrap_port {
     truncate(PORTS, 0);
     print PORTS "$_\n" for sort {$a <=> $b} keys %ports;
     close(PORTS) or die "Can't close ports file: $!";
+    push @ports, $port;
+    return $port;
 }
 
 sub bootstrap_tempdir {
@@ -301,7 +305,7 @@ sub bootstrap_config {
     open( my $config, '>', $tmp{'config'}{'RT'} )
         or die "Couldn't open $tmp{'config'}{'RT'}: $!";
 
-    my $dbname = $ENV{RT_TEST_PARALLEL}? "rt4test_$port" : "rt4test";
+    my $dbname = $ENV{RT_TEST_PARALLEL}? "rt5test_$port" : "rt5test";
     print $config qq{
 Set( \$WebDomain, "localhost");
 Set( \$WebPort,   $port);
@@ -320,6 +324,10 @@ Set( \$ShowHistory, "always");
     if ( $ENV{'RT_TEST_DB_HOST'} ) {
         print $config "Set( \$DatabaseHost , '$ENV{'RT_TEST_DB_HOST'}');\n";
     }
+    if ( $ENV{'RT_TEST_RT_HOST'} ) {
+        # Used to add rights for test users in the DB when testing mysql/mariadb
+        print $config "Set( \$DatabaseRTHost , '$ENV{'RT_TEST_RT_HOST'}');\n";
+    }
 
     if ( $args{'plugins'} ) {
         print $config "Set( \@Plugins, qw(". join( ' ', @{ $args{'plugins'} } ) .") );\n";
@@ -363,6 +371,7 @@ END
 
     print $config "\n1;\n";
     $ENV{'RT_SITE_CONFIG'} = $tmp{'config'}{'RT'};
+    $ENV{'RT_SITE_CONFIG_DIR'} = '/dev/null';
     close $config;
 
     return $config;
@@ -411,11 +420,11 @@ sub set_config_wrapper {
         open( my $fh, '<', $tmp{'config'}{'RT'} )
             or die "Couldn't open config file: $!";
         my @lines;
-        while (<$fh>) {
-            if (not @lines or /^Set\(/) {
-                push @lines, $_;
+        while (my $line = <$fh>) {
+            if (not @lines or $line =~ /^Set\(/) {
+                push @lines, $line;
             } else {
-                $lines[-1] .= $_;
+                $lines[-1] .= $line;
             }
         }
         close $fh;
@@ -429,6 +438,7 @@ sub set_config_wrapper {
         # Format the new value for output
         require Data::Dumper;
         local $Data::Dumper::Terse = 1;
+        local $Data::Dumper::Deparse = 1;
         my $dump = Data::Dumper::Dumper([@_[2 .. $#_]]);
         $dump =~ s/;?\s+\Z//;
         push @lines, "Set( ${sigil}${name}, \@{". $dump ."});\n";
@@ -454,11 +464,14 @@ sub set_config_wrapper {
         # configuration that should be written.  This is necessary
         # because some extensions (RTIR, for example) temporarily swap
         # configuration values out and back in Mason during requests.
+
+        # No need to write configs for Set calls from LoadConfig methods
+        # like RT::Config::LoadConfigFromDatabase.
         my @caller = caller(1); # preserve list context
         @caller = caller(0) unless @caller;
 
         return RT::Config::WriteSet(@_)
-            if ($caller[1]||'') =~ /\.t$/;
+            if ( $caller[1] || '' ) =~ /\.t$/ && ( $caller[3] || '' ) !~ /LoadConfig/;
 
         return $old_sub->(@_);
     };
@@ -497,7 +510,7 @@ sub bootstrap_db {
     my $db_type = RT->Config->Get('DatabaseType');
 
     if ($db_type eq "SQLite") {
-        RT->Config->WriteSet( DatabaseName => File::Spec->catfile( $self->temp_directory, "rt4test" ) );
+        RT->Config->WriteSet( DatabaseName => File::Spec->catfile( $self->temp_directory, "rt5test" ) );
     }
 
     __create_database();
@@ -1091,7 +1104,7 @@ sub add_rights {
 
 =head2 switch_templates_to TYPE
 
-This runs /opt/rt4/etc/upgrade/switch-templates-to in order to change the templates from
+This runs /opt/rt5/etc/upgrade/switch-templates-to in order to change the templates from
 HTML to text or vice versa.  TYPE is the type to switch to, either C<html> or
 C<text>.
 
@@ -1690,7 +1703,15 @@ sub stop_server {
     kill 'TERM', @SERVERS;
     foreach my $pid (@SERVERS) {
         if ($ENV{RT_TEST_WEB_HANDLER} =~ /^apache/) {
-            sleep 1 while kill 0, $pid;
+            my $count = 0;
+            while ( kill 0, $pid ) {
+                sleep 1;
+                last if $count++ >= 100;
+            }
+
+            # Give it a final shot and leave it.
+            # It's more important to not hang the tests
+            kill 'KILL', $pid if kill 0, $pid;
         } else {
             waitpid $pid, 0;
         }
@@ -1739,7 +1760,9 @@ sub parse_mail {
     require RT::EmailParser;
     my $parser = RT::EmailParser->new;
     $parser->ParseMIMEEntityFromScalar( $mail );
-    return $parser->Entity;
+    my $entity = $parser->Entity;
+    $entity->{__store_link_to_object_to_avoid_early_cleanup} = $parser;
+    return $entity;
 }
 
 sub works {
@@ -1769,9 +1792,52 @@ sub done_testing {
     Test::NoWarnings::had_no_warnings();
     $check_warnings_in_end = 0;
 
+    if ($RT::Test::Web::INSTANCES) {
+        my $cleanup = RT::Test::Web->new;
+        undef $RT::Test::Web::INSTANCES;
+        $cleanup->no_warnings_ok;
+    }
+
     $builder->done_testing(@_);
 }
 
+# Some utilities must only run one process at a time, so they check
+# for other running processes and quit if another is found. In parallel
+# test mode, this can cause test failures if two tests happen to run at
+# the same time.
+
+# This test helper checks for a running process, and if found sleeps and
+# tries again for a short time rather than immediately failing.
+
+sub run_singleton_command {
+    my $self    = shift;
+    my $command = shift;
+    my @args    = @_;
+
+    my $dir = "$tmp{'directory'}/../singleton";
+    mkdir $dir unless -e $dir;
+
+    my $flag = $command;
+    $flag =~ s!/!-!g;
+    $flag = "$dir/$flag";
+
+    for ( 1 .. 100 ) {
+        if ( -e $flag ) {
+            sleep 1;
+        }
+        else {
+            open my $fh, '>', $flag or die $!;
+            close $fh;
+            last;
+        }
+    }
+
+    my $ret = !system( $command, @args );
+    unlink $flag;
+
+    return $ret;
+}
+
 END {
     my $Test = RT::Test->builder;
     return if $Test->{Original_Pid} != $$;
@@ -1800,7 +1866,7 @@ END {
 
     # Drop our port from t/tmp/ports; do this after dropping the
     # database, as our port lock is also a lock on the database name.
-    if ($port) {
+    if (@ports) {
         my %ports;
         my $portfile = "$tmp{'directory'}/../ports";
         sysopen(PORTS, $portfile, O_RDWR|O_CREAT)
@@ -1808,7 +1874,7 @@ END {
         flock(PORTS, LOCK_EX)
             or die "Can't write-lock ports file $portfile: $!";
         $ports{$_}++ for split ' ', join("",<PORTS>);
-        delete $ports{$port};
+        delete $ports{$_} for @ports;
         seek(PORTS, 0, 0);
         truncate(PORTS, 0);
         print PORTS "$_\n" for sort {$a <=> $b} keys %ports;
